/*******************************************************************************
 * XUL namespace constant
 ******************************************************************************/
const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

/*******************************************************************************
 * Component constants
 ******************************************************************************/
const Cc = Components.classes;
const Ci = Components.interfaces;

/*******************************************************************************
 * Flickrsidebar controller object
 ******************************************************************************/
var gFlickrsidebar = null;
function start()
{
  gFlickrsidebar = new Flickrsidebar();
  gFlickrsidebar.start();
}

function stop()
{
  gFlickrsidebar.stop();
  gFlickrsidebar = null;
}

function Flickrsidebar() {}
Flickrsidebar.prototype = {
  _branch: null,
  _bundle: null,
  _request: null,
  _frob: null,
  _current: 0,
  _total: 0,
  _timer: null,
  _photos: null,
  _index: null,
  _history: null,
  _ignoreFocus: false,
  _ignoreClick: false, 
  
  get branch() {
    if (!this._branch) {
      var pbs = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService);
      this._branch = pbs.getBranch("extensions.flickrsidebar."); 
    }
    return this._branch;
  },
  
  get bundle() {
    if (!this._bundle)
      this._bundle = document.getElementById("mainBundle");    
    return this._bundle;  
  },
 
  get token() {
    var value = this.branch.getComplexValue("api.token", Ci.nsISupportsString).data;
    if (!value)
      return null;
    else
      return value;  
  },
 
  set token(aValue) {
    var comp = Cc["@mozilla.org/supports-string;1"].createInstance(Ci.nsISupportsString);
    comp.data = aValue;
    this.branch.setComplexValue("api.token", Ci.nsISupportsString, comp);
  },
  
  get url() {
    var value = this.branch.getComplexValue("api.url", Ci.nsISupportsString).data;
    if (!value)
      return null;
    else
      return value;  
  },
 
  get key() {
    var value = this.branch.getComplexValue("api.key", Ci.nsISupportsString).data;
    if (!value)
      return null;
    else
      return value;  
  },
 
  get secret() {
    var value = this.branch.getComplexValue("api.secret", Ci.nsISupportsString).data;
    if (!value)
      return null;
    else
      return value;  
  },
  
  get priority() {
    return this.branch.getIntPref("photos.priority");
  },
  
  get timeout() {
    return this.branch.getIntPref("photos.timeout");
  },
  
  get count() {
    return this.branch.getIntPref("photos.count");
  },
  
  get size() {
    return this.branch.getCharPref("photos.size");
  },
     
  get nsid() {
    var value = this.branch.getComplexValue("photos.nsid", Ci.nsISupportsString).data;
    if (!value)
      return null;
    else
      return value;  
  },
 
  set nsid(aValue) {
    var comp = Cc["@mozilla.org/supports-string;1"].createInstance(Ci.nsISupportsString);
    comp.data = aValue;
    this.branch.setComplexValue("photos.nsid", Ci.nsISupportsString, comp);
  },
  
  get group() {
    return this.branch.getBoolPref("photos.group");
  },
  
  set group(aValue) {
    this.branch.setBoolPref("photos.group", aValue);
  },
                
  start: function Flickrsidebar_start()
  {
    //hookup history
    try {
      this._history = Cc["@mozilla.org/satchel/form-history;1"].getService(Ci.nsIFormHistory2);
    } catch(e) {
      this._history = Cc["@mozilla.org/satchel/form-history;1"].getService(Ci.nsIFormHistory);
    }
    
    //check that we have the need api values
    if (!this.checkAPI()) {
      this.setMessage("api");
      return;
    }

    //add preference observer
    var pbi = this.branch.QueryInterface(Ci.nsIPrefBranchInternal);
    pbi.addObserver("", this, false);
        
    //request frob if token doesn't exist
    if (!this.token)
      this.getFrob();   
    else
      this.checkToken();
  },
  
  stop: function Flickrsidebar_stop()
  {
    //cancel pending requests
    this.cancelRequests();
    
    //remove preference observer
    try {
      var pbi = this._branch.QueryInterface(Ci.nsIPrefBranchInternal);
      pbi.removeObserver("", this);
    } catch(e) {}
        
    //remove variables;
    this._branch = null;
    this._bundle = null;
    this._request = null;
    this._frob = null;
    this._current = null;
    this._total = null;
    this._timer = null;
    this._photos = null;
    this._index = null;
    this._history = null;
    this._ignoreFocus = null;
    this._ignoreClick = null;     
  },
  
  delayedStart: function Flickrsidebar_delayedStart()
  {           
    //determine stream to show
    var popup = document.getElementById("photosPopup");
    var item = null;
    var event = {};    
    for (var i=0; i< popup.childNodes.length; i++) {
      item = popup.childNodes[i];
      if (item.getAttribute("nsid") != this.nsid)
        continue;
        
      //found item
      item.setAttribute("checked", "true");
      event.originalTarget = item;
      this.showPhotos(event);
      return;
    }
                    
    //selected stream could not be found
    //so select everyones by default
    item = document.getElementById("everyones");
    item.setAttribute("checked", "true");
    event.originalTarget = item;
    this.showPhotos(event);
  },
    
  openOptions: function Flickrsidebar_openOptions()
  {
    //get options window
    var windowManager = Cc['@mozilla.org/appshell/window-mediator;1'].getService(Ci.nsIWindowMediator);
    var options = windowManager.getMostRecentWindow("flickrsidebar:options");

    //focus window
    if (options)
      options.focus();
    else
      window.open("chrome://flickrsidebar/content/options.xul", "_blank", "chrome,modal,dialog");
  },
  
  openPhoto: function Flickrsidebar_openPhoto(aEvent, aWhere)
  {
    //nothing selected
    var item = document.getElementById("photosBox").selectedItem;
    if (!item)
      return;
          
    //get where value
    if (!aWhere)
      aWhere = whereToOpenLink(aEvent, false, false);
      
    //open link
    openUILinkIn(item.getAttribute("url"), aWhere);  
  },
    
  checkMiddleClick: function Flickrsidebar_checkMiddleClick(aEvent)
  {
    //nothing selected
    var item = document.getElementById("photosBox").selectedItem;
    if (!item)
      return;
    
    //middle click  
    if (aEvent.button == 1)  
      this.openPhoto(aEvent);    
  },
    
  showMorePhotos: function Flickrsidebar_showMorePhotos(aIncrement)
  {    
    this._current = this._current + aIncrement;
	  this.getPhotos();      
  },
  
  showPhotos: function Flickrsidebar_showPhotos(aEvent)
  {
    //reset page variables
    this._current = 1;
    this._total = 1;   
    
    //get stream to search
    if (aEvent) {
      var item = aEvent.originalTarget;
      var search = document.getElementById("searchBox");
      search.setAttribute("tooltiptext", item.getAttribute("label"));
      this.nsid = item.getAttribute("nsid");
      this.group = (item.getAttribute("group") == "true") ? true : false;
    }
	  this.getPhotos();               
  },
  
  getPhotos: function Flickrsidebar_getPhotos()
  {
    //cancel previous request
    this.cancelRequests();
    
    //set throbber and message
    this.setThrobber(true);
    this.updateUI(true);
	
	  //send request
	  this.setRequest(this.buildURL(), "getPhotos");      
  },  
  
  updateContext: function Flickrsidebar_updateContext()
  {
    var item = document.getElementById("photosBox").selectedItem;
    var open = document.getElementById("cmdOpen");
    var opentab = document.getElementById("cmdOpenTab");    
    var openwindow = document.getElementById("cmdOpenWindow");
      
    //set disabled    
    if (!item) {
      open.setAttribute("disabled", "true");
      opentab.setAttribute("disabled", "true");
      openwindow.setAttribute("disabled", "true"); 
    } else {
      open.removeAttribute("disabled");
      opentab.removeAttribute("disabled"); 
      openwindow.removeAttribute("disabled");
    }
  },
    
  checkToken: function Flickrsidebar_checkToken()
  {
    //set throbber and message
    this.setThrobber(true);    
    this.updateUI(false);
    
    //create url
	  var signed = MD5(this.secret + "api_key" + this.key + "auth_token" + this.token + "method" + "flickr.auth.checkToken");
	  var url = this.url + "?method=flickr.auth.checkToken" + "&api_key=" + this.key + "&auth_token=" + this.token + "&api_sig=" + signed;
	
	  //send request
	  this.setRequest(url, "checkToken");
  },
  
  getFrob: function Flickrsidebar_getFrob()
  {
    //set throbber and message
    this.setThrobber(true);    
    this.updateUI(false);
    
    //create url and signature
	  var signed = MD5(this.secret + "api_key" + this.key + "method" + "flickr.auth.getFrob");
	  var url = this.url + "?method=flickr.auth.getFrob" + "&api_key=" + this.key + "&api_sig=" + signed;

    //send request
    this.setRequest(url, "getFrob");   
  },

  getToken: function Flickrsidebar_getToken()
  {    
    //set throbber and message
    this.setThrobber(true);    
    this.updateUI(false);
    
    //create url and signature
    var signed = MD5(this.secret + "api_key" + this.key + "frob" + this._frob + "method" + "flickr.auth.getToken");
    var url = this.url + "?method=flickr.auth.getToken" + "&api_key=" + this.key + "&frob=" + this._frob + "&api_sig=" + signed;

    //send request
    this.setRequest(url, "getToken");    
  },
    
  getContacts: function Flickrsidebar_getContacts()
  {
    //set throbber and message
    this.setThrobber(true);    
    this.updateUI(false);
        
    //create url
    var signed = MD5(this.secret + "api_key" + this.key + "auth_token" + this.token + "method" + "flickr.contacts.getList");
    var url = this.url + "?method=flickr.contacts.getList" + "&api_key=" + this.key + "&auth_token=" + this.token + "&api_sig=" + signed;
	
    //send request
    this.setRequest(url, "getContacts");  
  },
  
  getGroups: function Flickrsidebar_getGroups()
  {
    //set throbber and message
    this.setThrobber(true);    
    this.updateUI(false);
        
    //create url
    var signed = MD5(this.secret + "api_key" + this.key + "auth_token" + this.token + "method" + "flickr.groups.pools.getGroups");
    var url = this.url + "?method=flickr.groups.pools.getGroups" + "&api_key=" + this.key + "&auth_token=" + this.token + "&api_sig=" + signed;
	
	//send request
	this.setRequest(url, "getGroups");  
  },
  
  onLoad: function  Flickrsidebar_onLoad()
  {
    //stop timer and throbber
    this.setThrobber(false);
    this._request.timer.cancel();
    try {
      window.clearTimeout(this._timer);
    } catch(e) {}
    
    //check server response
    try {
      var status = this._request.status;
    } catch (e) {
      status = -1;
    }
      
    switch (status) {
      case 407:
        this.setMessage("proxy");
        return;
      case -1:
        this.setMessage("contact");
        return;
      default:
        if (status > 200) {
          this.setMessage("status");
          return;
        }
    }
         
    //check response document
    if (this._request.responseText == "") {
      this.setMessage("response");
      return;
    }
    if (this._request.responseXML.childNodes[0].localName == "parsererror") {
      this.setMessage("response");
      return;    
    }
              
    //call finish function
    switch (this._request.callback) {
      case "checkToken":
        this.onCheckToken();
        break;    
      case "getFrob":
        this.onGetFrob();
        break;
      case "getToken":
        this.onGetToken();
        break;
      case "getContacts":
        this.onGetContacts();
        break;        
      case "getGroups":
        this.onGetGroups();
        break;
      case "getPhotos":
        this.onGetPhotos();
        break;                 
    }
  },
        
  onCheckToken: function Flickrsidebar_onCheckToken()
  {
    //parse response
    var response = this._request.responseXML.getElementsByTagName("rsp")[0];
    var state = response.getAttribute("stat");
    if (state == "fail") {
      var err = response.getElementsByTagName("err")[0];
      var code = err.getAttribute("code");
      if (code == "98")
        this.getFrob();
      else
        this.setMessage("error");
      return;
    }
    
    //set token
    var tokenEl = response.getElementsByTagName("token")[0];
    var token = tokenEl.firstChild.nodeValue;
    this.token = token;
        
    //set nsid 
    var user = response.getElementsByTagName("user")[0];
    var nsid = user.getAttribute("nsid");
    var yours = document.getElementById("yours");
    yours.setAttribute("nsid", nsid);
        
    //load contacts
    this.getContacts();
  },
  
  onGetFrob: function Flickrsidebar_onGetFrob()
  {
    //parse response
    var response = this._request.responseXML.getElementsByTagName("rsp")[0];
    var state = response.getAttribute("stat");
    if (state == "fail") {
      this.setMessage("error");
      return;
    }
    
    //create url to visit
    var frobEl = response.getElementsByTagName("frob")[0];
    this._frob = frobEl.firstChild.nodeValue;
    var signed = MD5(this.secret + "api_key" + this.key + "frob" + this._frob + "permsread");
    var url = "http://flickr.com/services/auth/?api_key=" + this.key + "&perms=read" + "&frob=" + this._frob + "&api_sig=" + signed;

    //opening a new focused tab.
    var browser = window.parent.getBrowser();    
    var tab = browser.addTab(url);
    browser.selectedTab = tab;
    
    //set authentication message
    this.setMessage("auth");
  },

  onGetToken: function Flickrsidebar_onGetToken()
  {
    //parse response
    var response = this._request.responseXML.getElementsByTagName("rsp")[0];
    var state = response.getAttribute("stat");
    if (state == "fail") {
      this.setMessage("error");
      return;
    }
    
    //set token
    var tokenEl = response.getElementsByTagName("token")[0];
    var token = tokenEl.firstChild.nodeValue;
    this.token = token;
    
    //set nsid 
    var user = response.getElementsByTagName("user")[0];
    var nsid = user.getAttribute("nsid");
    var yours = document.getElementById("yours");
    yours.setAttribute("nsid", nsid);
    
    //load contacts
    this.getContacts();  
  },
  
  onGetContacts: function Flickrsidebar_onGetContact()
  {
    //get elements
    var popup = document.getElementById("photosPopup");
    var separator = document.getElementById("contactsSeparator");
    var groups = document.getElementById("groupsSeparator"); 
    
    //hide contacts seperator
    separator.setAttribute("hidden", "true");
    
    //parse response
    var response = this._request.responseXML.getElementsByTagName("rsp")[0];
    var state = response.getAttribute("stat");
    if (state == "fail") {
      this.getGroups();
      return;
    }
    
    //build contact list
    var contacts = response.getElementsByTagName("contact");
    for (i=0; i<contacts.length; i++) {
      var contact = contacts[i];
      item = document.createElementNS(XUL_NS, "menuitem");
      if (contact.getAttribute("realname") != "")
        item.setAttribute("label", contact.getAttribute("realname"));
      else
        item.setAttribute("label", contact.getAttribute("username"));
      item.setAttribute("nsid", contact.getAttribute("nsid"));
      item.setAttribute("group", "false");
      item.setAttribute("type",  "radio");
      popup.insertBefore(item, groups);      
    }
    
    //show contacts separator
    if (contacts.length > 0)
      separator.removeAttribute("hidden");
      
    //get groups
    this.getGroups();        
  },
  
  onGetGroups: function Flickrsidebar_onGetGroups()
  {
    //get elements
    var popup = document.getElementById("photosPopup");
    var separator = document.getElementById("groupsSeparator");
    
    //hide groups seperator
    separator.setAttribute("hidden", "true");

    //parse response
    var response = this._request.responseXML.getElementsByTagName("rsp")[0];
    var state = response.getAttribute("stat");
    if (state == "fail") {
      this.delayedStart();
      return;
    }
    
    //build group list
    var groups = response.getElementsByTagName("group");
    for (i=0; i<groups.length; i++) {
      var group = groups[i];
      item = document.createElementNS(XUL_NS, "menuitem");
      if (group.getAttribute("name") != "")
        item.setAttribute("label", group.getAttribute("name"));
      else
        item.setAttribute("label", group.getAttribute("nsid"));      
      item.setAttribute("nsid", group.getAttribute("nsid"));
      item.setAttribute("group", "true");
      item.setAttribute("search", "false");      
      item.setAttribute("type",  "radio");
      popup.appendChild(item);      
    }
    
    //show groups separator
    if (groups.length > 0)
      separator.removeAttribute("hidden");
      
    //show first photos
    this.delayedStart();          
  },
  
  onGetPhotos: function Flickrsidebar_onGetPhotos()
  { 
    //parse response
    var response = this._request.responseXML.getElementsByTagName("rsp")[0];
    var state = response.getAttribute("stat");
    if (state == "fail") {
      this.setMessage("error");
      return;
    }
    
    //clear old images
	  var box = document.getElementById("photosBox");
	  while (box.hasChildNodes())
	    box.removeChild(box.lastChild);
  
    //set current and total pages
    var photosEl = response.getElementsByTagName("photos")[0];
    this._current = parseInt(photosEl.getAttribute("page"));
    this._total = parseInt(photosEl.getAttribute("pages"));
    
    //update UI
    this.updateUI(true);
    
    //start displaying images
    this._photos = response.getElementsByTagName("photo");
    this._index = 0;
    this.addPhoto();
  },
   
  setThrobber: function Flickrsidebar_setThrobber(aLoading)
  {
    var throbber = window.parent.document.getElementById("sidebar-throbber");
    if (aLoading)
      throbber.setAttribute("loading", "true");
    else
      throbber.removeAttribute("loading");
  },
  
  setMessage: function Flickrsidebar_setMessage(aType, aNode)
  {
    //get elements
    var title = document.getElementById("messageTitle");
    var description = document.getElementById("messageDescription"); 
    var button = document.getElementById("messageButton");
    var deck = document.getElementById("contentDeck");
    
    //remove previous description
    while (description.hasChildNodes())
      description.removeChild(description.lastChild);
    
    //set new message attributes
    title.setAttribute("value", this.bundle.getString(aType + ".title"));
    if (aType != "error" && aType != "status")
      description.appendChild(document.createTextNode(this.bundle.getString(aType + ".description")));    
    button.setAttribute("label", this.bundle.getString("again.label"));
    button.setAttribute("accesskey", this.bundle.getString("again.accesskey")); 
    button.setAttribute("oncommand", "gFlickrsidebar." + this._request.callback + "();");
    
    //set specific overrides          
    switch (aType) {
      case "api":
        button.setAttribute("oncommand", "gFlickrsidebar.start();");
        break;
      case "error":
        var response = this._request.responseXML.getElementsByTagName("rsp")[0];
        var err = response.getElementsByTagName("err")[0];
        description.appendChild(document.createTextNode(err.getAttribute("msg")));
        break;
      case "status":
        var status = this.bundle.getFormattedString("status.description", [this._request.status]);
        description.appendChild(document.createTextNode(status));
        break;
      case "auth":
        button.setAttribute("label", this.bundle.getString("done.label"));
        button.setAttribute("accesskey", this.bundle.getString("done.accesskey"));
        button.setAttribute("oncommand", "gFlickrsidebar.getToken();");
        break;  
    }
    
    //show message
    deck.selectedIndex = 1;
  },
  
  setRequest: function Flickrsidebar_setRequest(aURL, aCallback)
  {
    var component = this;  
    this._request = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);
    this._request.timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
    this._request.overrideMimeType("text/xml");
    this._request.open("GET", aURL, true);   
    if (this._request.channel instanceof Ci.nsISupportsPriority)
      this._request.channel.priority = this.priority;
      
    this._request.callback = aCallback;
    this._request.onload = function() { component.onLoad(); };
    this._request.onerror = function() { component.onLoad(); };
    
    //fire request
    this._request.timer.initWithCallback(this, this.timeout * 1000, Ci.nsITimer.TYPE_ONE_SHOT);
    this._request.send(null); 
  },

  updateUI: function Flickrsidebar_updateUI(aEnabled)
  { 
    //get elements
    var deck = document.getElementById("contentDeck");
    var search = document.getElementById("searchBox");
    var select = document.getElementById("selectButton");
    var previous = document.getElementById("cmdPrevious");
    var next = document.getElementById("cmdNext");
    
    //enable ui
    if (aEnabled) {
      select.removeAttribute("disabled"); 
      
      //can not search everyones   
      if (this.nsid == "favorites")
        search.setAttribute("disabled", "true");
      else
        search.removeAttribute("disabled");
        
      //on first page
      if (this._current > 1) {
        previous.removeAttribute("disabled");
        previous.setAttribute("tooltiptext", this.bundle.getFormattedString("page.tooltip", [this._current - 1, this._total]));
      } else {
        previous.setAttribute("disabled", "true");
        previous.setAttribute("tooltiptext", this.bundle.getString("previous.tooltip"));
      }
        
      //on last page
      if (this._current < this._total) {
        next.removeAttribute("disabled");
        next.setAttribute("tooltiptext", this.bundle.getFormattedString("page.tooltip", [this._current + 1, this._total]));
      } else {
        next.setAttribute("disabled", "true");
        next.setAttribute("tooltiptext", this.bundle.getString("next.tooltip"));
      }
        
    //disable ui   
    } else {
      search.setAttribute("disabled", "true");
      select.setAttribute("disabled", "true");      
      previous.setAttribute("disabled", "true");
      next.setAttribute("disabled", "true");
      previous.setAttribute("tooltiptext", this.bundle.getString("previous.tooltip"));
      next.setAttribute("tooltiptext", this.bundle.getString("next.tooltip"));
    }
    
    //show photo box
    deck.selectedIndex = 0;
  },
  
  buildURL: function Flickrsidebar_buildURL()
  {
	  //build url	  
    var parms = [];
    var values = {};  
	  var search = this.cleanupSearch();  
    var method = "";  	    
    if (this.nsid == "everyones" && !search)
      method = "flickr.photos.getRecent";       
    else if (this.nsid == "everyones")
      method = "flickr.photos.search";
    else if (this.nsid == "favorites")
      method = "flickr.favorites.getList";
    else {
      if (this.group) {
        method = "flickr.groups.pools.getPhotos";
        parms.push("group_id");
        values["group_id"] = this.nsid;
      } else {
        method = "flickr.photos.search";
        parms.push("user_id");
        values["user_id"] = this.nsid;
      }
    }
    
    //set the parms
    parms.push("api_key");
    values["api_key"] = this.key;
    parms.push("auth_token");
    values["auth_token"] = this.token;
    parms.push("per_page");
    values["per_page"] = this.count;
    parms.push("page");
    values["page"] = this._current;
    parms.push("method");
    values["method"] = method;
    
    //set tags
    if (search && this.nsid != "favorites") {
      parms.push("tags");
      values["tags"] = search;
    }

    //build the url
    parms.sort();
    var signed = this.secret;
    for (var i=0; i<parms.length; i++)
      signed += parms[i] + values[parms[i]];
    signed = MD5(signed);  

    var url = this.url + "?method=" + method;
    for (i=0; i<parms.length; i++) {
      if (parms[i] == "method")
        continue;
      url += "&" + parms[i] + "=" + escape(values[parms[i]]);
    }
    url += "&api_sig=" + signed;
    
    return url;  
  },
  
  addPhoto: function Flickrsidebar_addPhoto()
  {
    //no photos in stream
    if (this._photos.length <= 0) {
      this.setThrobber(false);
      return;
    }
    
    //all photos displayed
    if (this._index >= this._photos.length) {
      this.setThrobber(false);
      return;
    }
    
    //create placeholders
    var box = document.getElementById("photosBox");
    var item = null;
    if (this._index == 0) {
      this.setThrobber(true);
      for (var x=0; x<this._photos.length; x++) {
        item = this.createPhoto(this._photos[x]);
        box.appendChild(item); 
      }
    }
    
    //display the next photo
    this.setThrobber(true);
    item = box.getItemAtIndex(this._index);
    item.firstChild.setAttribute("src", item.getAttribute("imgurl"));

    //set timeout to show photo
    ++this._index;
    this._timer = window.setTimeout("gFlickrsidebar.addPhoto();", 20); 
  },
  
  createPhoto: function Flickrsidebar_createPhoto(aPhoto)
  {
    //get photo properties
    var server = aPhoto.getAttribute("server");
    var id = aPhoto.getAttribute("id");
    var secret = aPhoto.getAttribute("secret");
    var user = aPhoto.getAttribute("owner");
    var title = aPhoto.getAttribute("title");
            
    //create image
    var image = document.createElementNS(XUL_NS, "image");
    var imgurl = "http://static.flickr.com/" + server + "/" + id + "_" + secret + "_" + this.size + ".jpg";
    image.setAttribute("src", "resource://gre/res/loading-image.gif");
    image.setAttribute("validate", "never");
    image.setAttribute("tooltiptext", title);
        
    //create item
    var item = document.createElementNS(XUL_NS, "richlistitem");
    var url = "http://www.flickr.com/photos/" + user + "/" + id + "/";
    item.setAttribute("tooltiptext", title);
    item.setAttribute("url", url);
    item.setAttribute("pid", id);  
    item.setAttribute("imgurl", imgurl);
    item.appendChild(image);
    return item;  
  },
    
  checkAPI: function Flickrsidebar_checkAPI()
  {
    if (!this.url)
      return false;
      
    if (!this.key)
      return false;
       
    if (!this.secret)
      return false;
                 
    return true;
  },
  
  cancelRequests: function Flickrsidebar_cancelRequest()
  {
    //stop window timer
    try {
      window.clearTimeout(this._timer);
    } catch(e) {}
    
    //check request  
    if (!this._request)
      return;
      
    //abort request
    try {
      this._request.abort();
    } catch(e) {}
    
    //stop request timer
    try {
      this._request.timer.cancel();
    } catch(e) {}
  },
  
  cleanupSearch: function Flickrsidebar_cleanupSearch()
  {
    //get search term
    var term = document.getElementById("searchBox").value;

    //replace spaces with commas
    if (term) {
      try {
        this._history.addEntry("flickrsidebar", term);
      } catch(e) {}
      term = term.replace(/[\s]+/g, ",");
    }
      
    return term;  
  },
  
  ///////////////////////////
  // nsITimerCallback
  notify: function Flickrsidebar_notify(timer)
  {
    //stop throbber and request
    this.setThrobber(false);
    this._request.abort();
    try {
      window.clearTimeout(this._timer);
    } catch(e) {}
        
    //set message
    this.setMessage("timeout");
  },

  /////////////////////////////
  // nsIObserver
  observe: function Flickrsidebar_observe(aSubject, aTopic, aData)
  {
    //only worry about display options
    if (aData != "photos.size" && aData !="photos.count")
      return;
    
    //reload display  
    window.setTimeout("gFlickrsidebar.getPhotos();", 20);
  },
       
  ///////////////////////////
  // nsISupports   
  QueryInterface: function Flickrsidebar_QueryInterface(aIID)
  {       
    if (!aIID.equals(Ci.nsISupports) &&
        !aIID.equals(Ci.nsITimerCallback) &&
        !aIID.equals(Ci.nsIObserver))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};